@@ -1,7 +1,10 @@
 #!/usr/bin/env bash
 
 set -e
-set -x
+
+if [ -n "${EZ_VERBOSE}" ]; then
+    set -x
+fi
 
 # Warning: This variable API is experimental so these variables may be subject
 # to change in the future.
@@ -29,16 +32,18 @@ app_logdir=${app_logdir:=/var/log/puppetlabs/${real_name}}
 # EZBake Vars    #
 ##################
 
-set +x
-echo "#-------------------------------------------------#"
-echo "The following variables are set: "
-echo
-env | sort
+if [ -n "${EZ_VERBOSE}" ]; then
+    set +x
+    echo "#-------------------------------------------------#"
+    echo "The following variables are set: "
+    echo
+    env | sort
 
-echo
-echo "End of variable print."
-echo "#-------------------------------------------------#"
-set -x
+    echo
+    echo "End of variable print."
+    echo "#-------------------------------------------------#"
+    set -x
+fi
 
 ##################
 # Task functions #
@@ -201,7 +206,7 @@ function task_preinst_redhat {
 function task_preinst_deb {
     # Create the "puppet" user
     if ! getent passwd puppet > /dev/null; then
-        adduser --quiet --system --group --home ${datadir}/puppetserver  \
+        adduser --quiet --system --group --home "${app_data}"  \
         --no-create-home                                 \
         --gecos "puppetserver daemon" \
         puppet ; \
@@ -218,6 +223,7 @@ function task_preinst_deb {
 # Debian based post-installation tasks.
 function task_postinst_deb {
     task postinst_permissions
+    install --owner=puppet --group=puppet -d /opt/puppetlabs/server/data/puppetserver/jruby-gems
     /opt/puppetlabs/puppet/bin/puppet config set --section master vardir  /opt/puppetlabs/server/data/puppetserver
     /opt/puppetlabs/puppet/bin/puppet config set --section master logdir  /var/log/puppetlabs/puppetserver
     /opt/puppetlabs/puppet/bin/puppet config set --section master rundir  /var/run/puppetlabs/puppetserver
@@ -233,6 +239,7 @@ function task_postinst_deb {
 # RPM based post-installation tasks.
 function task_postinst_redhat {
     : # Null command in case additional_postinst is empty
+    install --owner=puppet --group=puppet -d /opt/puppetlabs/server/data/puppetserver/jruby-gems
     /opt/puppetlabs/puppet/bin/puppet config set --section master vardir  /opt/puppetlabs/server/data/puppetserver
     /opt/puppetlabs/puppet/bin/puppet config set --section master logdir  /var/log/puppetlabs/puppetserver
     /opt/puppetlabs/puppet/bin/puppet config set --section master rundir  /var/run/puppetlabs/puppetserver
@@ -271,16 +278,23 @@ function task_logrotate_legacy {
 # Misc functions #
 ##################
 
+# Print output only if EZ_VERBOSE is set
+function debug_echo {
+    if [ -n "${EZ_VERBOSE}" ]; then
+        echo $@
+    fi
+}
+
 # Do basic OS detection using facter.
 function osdetection {
     OSFAMILY=`facter osfamily`
     MAJREV=`facter operatingsystemmajrelease`
 
-    echo "OS Detection results"
-    echo
-    echo "OSFAMILY: ${OSFAMILY}"
-    echo "MAJREV: ${MAJREV}"
-    echo
+    debug_echo "OS Detection results"
+    debug_echo
+    debug_echo "OSFAMILY: ${OSFAMILY}"
+    debug_echo "MAJREV: ${MAJREV}"
+    debug_echo
 }
 
 # Run a task
@@ -289,7 +303,7 @@ function osdetection {
 function task {
     local task=$1
     shift
-    echo "Running task ${task} ..."
+    debug_echo "Running task ${task} ..."
     eval task_$task $@
 }
 
