@@ -1,10 +1,15 @@
 (ns puppetlabs.services.jruby.jruby-puppet-core
-  (:require [schema.core :as schema]
+  (:require [clojure.tools.logging :as log]
+            [me.raynes.fs :as fs]
+            [schema.core :as schema]
+            [puppetlabs.kitchensink.classpath :as ks-classpath]
             [puppetlabs.kitchensink.core :as ks]
             [puppetlabs.services.jruby.jruby-puppet-schemas :as jruby-schemas]
             [puppetlabs.services.jruby.puppet-environments :as puppet-env]
             [puppetlabs.services.jruby.jruby-puppet-internal :as jruby-internal]
-            [puppetlabs.services.jruby.jruby-puppet-agents :as jruby-agents])
+            [puppetlabs.services.jruby.jruby-puppet-agents :as jruby-agents]
+            [clojure.java.io :as io]
+            [clojure.tools.logging :as log])
   (:import (puppetlabs.services.jruby.jruby_puppet_schemas JRubyPuppetInstance)
            (com.puppetlabs.puppetserver PuppetProfiler)))
 
@@ -26,20 +31,20 @@
   be available on the socket. Currently set to 20 minutes."
   (* 20 60 1000))
 
-;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
-;;; Definitions
+(def default-master-conf-dir
+  "/etc/puppetlabs/puppet")
 
-(def jruby-puppet-env
-  "The environment variables that should be passed to the Puppet JRuby interpreters.
-  We don't want them to read any ruby environment variables, like $GEM_HOME or
-  $RUBY_LIB or anything like that, so pass it an empty environment map - except -
-  Puppet needs HOME and PATH for facter resolution, so leave those."
-  (select-keys (System/getenv) ["HOME" "PATH"]))
+(def default-master-code-dir
+  "/etc/puppetlabs/code")
 
-(def ruby-code-dir
-  "The name of the directory containing the ruby code in this project.
-  This directory lives under src/ruby/"
-  "puppet-server-lib")
+(def default-master-log-dir
+  "/var/log/puppetlabs/puppetserver")
+
+(def default-master-run-dir
+  "/var/run/puppetlabs/puppetserver")
+
+(def default-master-var-dir
+  "/opt/puppetlabs/server/data/puppetserver")
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;; Private
@@ -96,14 +101,33 @@
       (get-in config [:http-client :idle-timeout-milliseconds]
         default-http-socket-timeout))
     (update-in [:borrow-timeout] #(or % default-borrow-timeout))
-    (update-in [:master-conf-dir] #(or % nil))
-    (update-in [:master-var-dir] #(or % nil))
-    (update-in [:master-code-dir] #(or % nil))
-    (update-in [:master-run-dir] #(or % nil))
-    (update-in [:master-log-dir] #(or % nil))
+    (update-in [:master-conf-dir] #(or % default-master-conf-dir))
+    (update-in [:master-var-dir] #(or % default-master-var-dir))
+    (update-in [:master-code-dir] #(or % default-master-code-dir))
+    (update-in [:master-run-dir] #(or % default-master-run-dir))
+    (update-in [:master-log-dir] #(or % default-master-log-dir))
     (update-in [:max-active-instances] #(or % (default-pool-size (ks/num-cpus))))
     (update-in [:max-requests-per-instance] #(or % 0))))
 
+(def facter-jar
+  "Well-known name of the facter jar file"
+  "facter.jar")
+
+(schema/defn ^:always-validate
+  add-facter-jar-to-system-classloader
+  "Searches the ruby load path for a file whose name matches that of the
+  facter jar file.  The first one found is added to the system classloader's
+  classpath.  If no match is found, an info message is written to the log
+  but no failure is returned"
+  [ruby-load-path :- (schema/both (schema/pred vector?) [schema/Str]) ]
+  (if-let [facter-jar (first
+                        (filter fs/exists?
+                          (map #(fs/file % facter-jar) ruby-load-path)))]
+    (do
+      (log/debugf "Adding facter jar to classpath from: %s" facter-jar)
+      (ks-classpath/add-classpath facter-jar))
+    (log/info "Facter jar not found in ruby load path")))
+
 (schema/defn ^:always-validate
   create-pool-context :- jruby-schemas/PoolContext
   "Creates a new JRubyPuppet pool context with an empty pool. Once the JRubyPuppet
@@ -175,3 +199,24 @@
   "Return a borrowed pool instance to its free pool."
   [instance :- jruby-schemas/JRubyPuppetInstanceOrRetry]
   (jruby-internal/return-to-pool instance))
+
+(schema/defn ^:always-validate cli-ruby! :- jruby-schemas/JRubyMainStatus
+  "Run JRuby as though native `ruby` were invoked with args on the CLI"
+  [config :- {schema/Keyword schema/Any}
+   args :- [schema/Str]]
+  (let [main (jruby-internal/new-main (initialize-config config))
+        argv (into-array String (concat ["-rjar-dependencies"] args))]
+    (.run main argv)))
+
+(schema/defn ^:always-validate cli-run! :- (schema/maybe jruby-schemas/JRubyMainStatus)
+  "Run a JRuby CLI command, e.g. gem, irb, etc..."
+  [config :- {schema/Keyword schema/Any}
+   command :- schema/Str
+   args :- [schema/Str]]
+  (let [bin-dir "META-INF/jruby.home/bin"
+        load-path (format "%s/%s" bin-dir command)
+        url (io/resource load-path (.getClassLoader org.jruby.Main))]
+    (if url
+      (cli-ruby! config
+        (concat ["-e" (format "load '%s'" url) "--"] args))
+      (log/errorf "command %s could not be found in %s" command bin-dir))))
